@@ -1,31 +1,32 @@
 use num::Complex;
+use num::Zero;
 use std::rc::Rc;
 
-pub type Qubit = Vec<Complex<f64>>;
+pub type Complex64 = Complex<f64>;
 
-pub type Gate = Rc<Matrix>;
+pub type Qubit = Vec<Complex64>;
 
-pub type Matrix = Vec<Vec<Complex<f64>>>;
+pub type Gate = Vec<Vec<Complex64>>;
 
 pub type BinaryChars = Vec<char>;
 
 pub struct State {
     number_of_bit: u32,
     pub index: usize,
-    pub amp: Complex<f64>,
+    pub amp: Complex64,
     pub prob: f64,
 }
 
 impl State {
     pub fn to_binary_chars(&self, qb: &[u32]) -> BinaryChars {
-        let v: Vec<char> = to_binary_chars(self.index, self.number_of_bit as usize);
+        let v = to_binary_chars(self.index, self.number_of_bit as usize);
 
-        let mut out: Vec<char> = vec![];
+        let mut bin = vec![];
         for i in qb {
-            out.push(v[*i as usize]);
+            bin.push(v[*i as usize]);
         }
 
-        out
+        bin
     }
 }
 
@@ -67,7 +68,7 @@ impl Q {
     }
 
     pub fn zero_with(&mut self, n: u32) -> Vec<u32> {
-        let mut list: Vec<u32> = vec![];
+        let mut list = vec![];
 
         for _ in 0..n {
             list.push(self.zero());
@@ -77,12 +78,12 @@ impl Q {
     }
 
     pub fn zero_log2(&mut self, n: u32) -> Vec<u32> {
-        let log2n: u32 = ((n as f64).log2() as u32) + 1;
+        let log2n = ((n as f64).log2() as u32) + 1;
         self.zero_with(log2n)
     }
 
     fn tensor_vec(&mut self, qb: Qubit) {
-        let mut v: Qubit = vec![];
+        let mut v = vec![];
 
         for w in &self.qb {
             for j in &qb {
@@ -106,16 +107,18 @@ impl Q {
     }
 
     fn apply_(&mut self, g: Gate, qb: &[u32]) {
-        let list: Vec<Gate> = gate_list(self.number_of_bit(), g, qb);
-        let g: Gate = tensor_(&list);
+        let list = gate_list(self.number_of_bit(), g, qb);
+        let g = tensor_(&list);
         self.apply(g)
     }
 
+    #[allow(clippy::needless_range_loop)]
     pub fn apply(&mut self, g: Gate) {
-        let mut v: Qubit = vec![];
+        let mut v = vec![];
 
         for i in 0..g.len() {
             let mut e = Complex::new(0.0, 0.0);
+
             for j in 0..g[i].len() {
                 e += g[i][j] * self.qb[j];
             }
@@ -127,21 +130,21 @@ impl Q {
     }
 
     pub fn cmodexp2(&mut self, a: u32, n: u32, r0: &[u32], r1: &[u32]) {
-        let nob: u32 = self.number_of_bit();
+        let nob = self.number_of_bit();
         for (i, c) in r0.iter().enumerate() {
             self.apply(cmodexp2(nob, a, i as u32, n, *c, r1))
         }
     }
 
     pub fn iqft(&mut self, qb: &[u32]) {
-        let l: usize = qb.len();
+        let len = qb.len();
 
         // for i := l - 1; i > -1; i-- {}
-        for i in (0..l).rev() {
-            let mut k: i32 = (l as i32) - (i as i32);
+        for i in (0..len).rev() {
+            let mut k = (len - i) as i32;
 
             // for j := l - 1; j > i; j-- {}
-            for j in ((i + 1)..l).rev() {
+            for j in ((i + 1)..len).rev() {
                 self.icr(k, qb[j], qb[i]);
                 k -= 1;
             }
@@ -151,19 +154,18 @@ impl Q {
     }
 
     pub fn icr(&mut self, k: i32, control: u32, target: u32) {
-        let n: u32 = self.number_of_bit();
-        let g: Gate = dagger(cr(k, n, control, target));
+        let n = self.number_of_bit();
+        let g = dagger(cr(k, n, control, target));
         self.apply(g)
     }
 
     pub fn state(&self) -> Vec<State> {
-        let z: Complex<f64> = Complex::new(0.0, 0.0);
-        let nob: u32 = self.number_of_bit();
-        let mut list: Vec<State> = vec![];
+        let mut list = vec![];
+        let nob = self.number_of_bit();
 
         for i in 0..self.qb.len() {
-            let rqb: Complex<f64> = round(self.qb[i]);
-            if rqb == z {
+            let rqb = round(self.qb[i]);
+            if rqb.is_zero() {
                 continue;
             }
 
@@ -179,8 +181,8 @@ impl Q {
     }
 }
 
-fn tensor_(list: &[Gate]) -> Gate {
-    let mut g: Gate = Rc::clone(&list[0]);
+fn tensor_(list: &[Rc<Gate>]) -> Gate {
+    let mut g = list[0].to_vec();
 
     for i in list.iter().skip(1) {
         g = tensor(g, Rc::clone(i));
@@ -189,12 +191,13 @@ fn tensor_(list: &[Gate]) -> Gate {
     g
 }
 
-fn tensor(m: Gate, n: Gate) -> Gate {
-    let mut mat: Matrix = vec![];
+#[allow(clippy::needless_range_loop)]
+fn tensor(m: Gate, n: Rc<Gate>) -> Gate {
+    let mut g = vec![];
 
     for i in 0..m.len() {
         for k in 0..n.len() {
-            let mut v: Vec<Complex<f64>> = vec![];
+            let mut v = vec![];
 
             for j in 0..m[i].len() {
                 for l in 0..n[k].len() {
@@ -202,16 +205,17 @@ fn tensor(m: Gate, n: Gate) -> Gate {
                 }
             }
 
-            mat.push(v);
+            g.push(v);
         }
     }
 
-    Rc::new(mat)
+    g
 }
 
-fn gate_list(nob: u32, g: Gate, qb: &[u32]) -> Vec<Gate> {
-    let identity: Gate = id();
-    let mut list: Vec<Gate> = vec![];
+fn gate_list(nob: u32, g: Gate, qb: &[u32]) -> Vec<Rc<Gate>> {
+    let mut list = vec![];
+    let identity = Rc::new(id());
+    let rg = Rc::new(g);
 
     for i in 0..nob {
         let mut found = false;
@@ -224,7 +228,7 @@ fn gate_list(nob: u32, g: Gate, qb: &[u32]) -> Vec<Gate> {
         }
 
         if found {
-            list.push(Rc::clone(&g));
+            list.push(Rc::clone(&rg));
             continue;
         }
 
@@ -235,94 +239,94 @@ fn gate_list(nob: u32, g: Gate, qb: &[u32]) -> Vec<Gate> {
 }
 
 fn id() -> Gate {
-    Rc::new(vec![
+    vec![
         vec![Complex::new(1.0, 0.0), Complex::new(0.0, 0.0)],
         vec![Complex::new(0.0, 0.0), Complex::new(1.0, 0.0)],
-    ])
+    ]
 }
 
 fn x() -> Gate {
-    Rc::new(vec![
+    vec![
         vec![Complex::new(0.0, 0.0), Complex::new(1.0, 0.0)],
         vec![Complex::new(1.0, 0.0), Complex::new(0.0, 0.0)],
-    ])
+    ]
 }
 
 fn h() -> Gate {
     let e = Complex::new(1.0 / std::f64::consts::SQRT_2, 0.0);
-    Rc::new(vec![vec![e, e], vec![e, -1.0 * e]])
+    vec![vec![e, e], vec![e, -1.0 * e]]
 }
 
 fn cr(k: i32, nob: u32, control: u32, target: u32) -> Gate {
     // identity matrix
-    let mut mat: Matrix = idmat(nob);
+    let mut g = id_(nob);
 
     // coefficient
     let p = 2.0 * std::f64::consts::PI / (2.0_f64.powf(k as f64));
     let e = Complex::new(0.0, p).exp();
 
-    for (i, v) in mat.iter_mut().enumerate() {
-        let bits: BinaryChars = to_binary_chars(i, nob as usize);
+    for (i, v) in g.iter_mut().enumerate() {
+        let bits = to_binary_chars(i, nob as usize);
         if bits[control as usize] == '1' && bits[target as usize] == '1' {
             // apply
             v[i] = e * v[i];
         }
     }
 
-    transpose(Rc::new(mat))
+    transpose(g)
 }
 
 fn cmodexp2(nob: u32, a: u32, j: u32, n: u32, control: u32, target: &[u32]) -> Gate {
-    let r0len: u32 = nob - target.len() as u32;
-    let r1len: u32 = target.len() as u32;
+    let r0len = nob - target.len() as u32;
+    let r1len = target.len() as u32;
     let a2jmodn = super::number::modexp2(a, j, n);
 
-    let mut index: Vec<usize> = vec![];
+    let mut index = vec![];
     for i in 0..(2_usize.pow(nob)) {
-        let bits: BinaryChars = to_binary_chars(i, nob as usize);
+        let bits = to_binary_chars(i, nob as usize);
         if bits[control as usize] == '0' {
             // i -> i
-            index.push(to_decimal(&bits));
+            index.push(to_decimal(&bits) as usize);
             continue;
         }
 
-        let r1bits: BinaryChars = take(&bits, r0len as usize, bits.len());
-        let k: usize = to_decimal(&r1bits);
-        if (k as u32) > n - 1 {
+        let r1bits = take(&bits, r0len as usize, bits.len());
+        let k = to_decimal(&r1bits);
+        if k > n - 1 {
             // i -> i
-            index.push(to_decimal(&bits));
+            index.push(to_decimal(&bits) as usize);
             continue;
         }
 
         // i -> a**2**j *k mod n
-        let a2jkmodn: u32 = (a2jmodn * k as u32) % n;
-        let mut a2jkmodns: BinaryChars = to_binary_chars(a2jkmodn as usize, r1len as usize);
+        let a2jkmodn = (a2jmodn * k) % n;
+        let mut a2jkmodns = to_binary_chars(a2jkmodn as usize, r1len as usize);
 
-        let mut r0bits: BinaryChars = take(&bits, 0, r0len as usize);
+        let mut r0bits = take(&bits, 0, r0len as usize);
         r0bits.append(&mut a2jkmodns);
-        index.push(to_decimal(&r0bits));
+        index.push(to_decimal(&r0bits) as usize);
     }
 
-    let mat: Matrix = idmat(nob);
-    let mut out: Matrix = vec![vec![]; mat.len()];
+    let identity = id_(nob);
+    let mut g = vec![vec![]; identity.len()];
     for (i, ii) in index.iter().enumerate() {
-        out[i] = clone(&mat[*ii]); // :(
+        g[i] = clone_vec(&identity[*ii]);
     }
 
-    transpose(Rc::new(out))
+    transpose(g)
 }
 
-fn round(c: Complex<f64>) -> Complex<f64> {
-    let mut out: Complex<f64> = c;
+fn round(c: Complex64) -> Complex64 {
+    let mut round = c;
     if c.re.abs() < 1e-13 {
-        out.re = 0.0;
+        round.re = 0.0;
     }
 
     if c.im.abs() < 1e-13 {
-        out.im = 0.0;
+        round.im = 0.0;
     }
 
-    out
+    round
 }
 
 fn take(bin: &[char], start: usize, end: usize) -> BinaryChars {
@@ -333,16 +337,16 @@ fn to_binary_chars(i: usize, nob: usize) -> BinaryChars {
     format!("{:>0n$b}", i, n = nob).chars().collect()
 }
 
-fn to_decimal(v: &[char]) -> usize {
+fn to_decimal(v: &[char]) -> u32 {
     let s: String = v.iter().collect();
-    usize::from_str_radix(&s, 2).unwrap()
+    u32::from_str_radix(&s, 2).unwrap()
 }
 
-fn idmat(nob: u32) -> Matrix {
-    let mut mat: Matrix = vec![];
+fn id_(nob: u32) -> Vec<Vec<Complex64>> {
+    let mut mat = vec![];
 
     for i in 0..(2_i32.pow(nob)) {
-        let mut v: Vec<Complex<f64>> = vec![];
+        let mut v = vec![];
 
         for j in 0..(2_i32.pow(nob)) {
             if i == j {
@@ -359,56 +363,74 @@ fn idmat(nob: u32) -> Matrix {
     mat
 }
 
-fn clone(v: &[Complex<f64>]) -> Vec<Complex<f64>> {
-    let mut out: Vec<Complex<f64>> = vec![];
+fn clone_vec(v: &[Complex64]) -> Vec<Complex64> {
+    let mut clone = vec![];
+
     for i in v {
-        out.push(*i);
+        clone.push(*i);
     }
 
-    out
+    clone
 }
 
 fn dagger(g: Gate) -> Gate {
     transpose(conjugate(g))
 }
 
+#[allow(clippy::needless_range_loop)]
 fn transpose(g: Gate) -> Gate {
-    let mut mat: Matrix = vec![];
+    let mut trans = vec![];
 
     for i in 0..g.len() {
-        let mut v: Vec<Complex<f64>> = vec![];
+        let mut v = vec![];
 
         for j in 0..g[i].len() {
             v.push(g[j][i])
         }
 
-        mat.push(v);
+        trans.push(v);
     }
 
-    Rc::new(mat)
+    trans
 }
 
+#[allow(clippy::needless_range_loop)]
 fn conjugate(g: Gate) -> Gate {
-    let mut mat: Matrix = vec![];
+    let mut conj = vec![];
 
     for i in 0..g.len() {
-        let mut v: Vec<Complex<f64>> = vec![];
+        let mut v = vec![];
 
         for j in 0..g[i].len() {
             v.push(g[i][j].conj());
         }
 
-        mat.push(v);
+        conj.push(v);
     }
 
-    Rc::new(mat)
+    conj
+}
+
+#[test]
+fn test_to_binary_chars() {
+    assert_eq!(to_binary_chars(3, 5), vec!['0', '0', '0', '1', '1']);
+    assert_eq!(to_binary_chars(7, 5), vec!['0', '0', '1', '1', '1']);
+    assert_eq!(to_binary_chars(15, 5), vec!['0', '1', '1', '1', '1']);
+    assert_eq!(to_binary_chars(31, 5), vec!['1', '1', '1', '1', '1']);
+}
+
+#[test]
+fn test_to_decimal() {
+    assert_eq!(to_decimal(&['1']), 1);
+    assert_eq!(to_decimal(&['1', '1']), 3);
+    assert_eq!(to_decimal(&['1', '0', '1']), 5);
 }
 
 #[test]
 fn test_is_eigen_vector() {
-    let n: u32 = 15;
-    let a: u32 = 7;
-    let t: u32 = 3;
+    let n = 15;
+    let a = 7;
+    let t = 3;
 
     let mut qsim = Q::new();
     let r0 = qsim.zero_with(t);
@@ -419,12 +441,12 @@ fn test_is_eigen_vector() {
     qsim.cmodexp2(a, n, &r0, &r1);
     qsim.iqft(&r0);
 
-    let mut us: std::collections::HashMap<String, Complex<f64>> = std::collections::HashMap::new();
+    let mut us = std::collections::HashMap::new();
     for state in qsim.state().iter() {
-        let m1: Vec<char> = state.to_binary_chars(&r1);
+        let m1 = state.to_binary_chars(&r1);
         let ui: String = m1.iter().collect();
 
-        let v: Complex<f64> = match us.get(&ui) {
+        let v = match us.get(&ui) {
             Some(vv) => state.amp + vv,
             None => state.amp,
         };
@@ -432,15 +454,20 @@ fn test_is_eigen_vector() {
         us.insert(ui, v);
     }
 
-    for (m, v) in &us {
-        println!("{:?} {}", m, v);
+    let cases = vec![
+        ("0001", Complex::new(1.0, 0.0)),
+        ("0100", Complex::new(0.0, 0.0)),
+        ("0111", Complex::new(0.0, 0.0)),
+        ("1101", Complex::new(0.0, 0.0)),
+    ];
+
+    for (i, c) in cases {
+        let v = match us.get(i) {
+            Some(v) => *v,
+            None => panic!("{} not found", i),
+        };
+
+        assert!((v - c).re.abs() < 1e-13);
+        assert!((v - c).im.abs() < 1e-13);
     }
-
-    let v: Complex<f64> = match us.get("0001") {
-        Some(v) => *v,
-        None => panic!("0001 not found"),
-    };
-
-    assert!((v.re - 1.0) < 1e-13);
-    assert!((v.im - 0.0) < 1e-13);
 }
